package com.yame.uml.control;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.util.List;

import com.yame.uml.model.Result;
import com.yame.uml.model.ResultType;

import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import lombok.extern.slf4j.Slf4j;
import net.sourceforge.plantuml.BlockUml;
import net.sourceforge.plantuml.FileFormat;
import net.sourceforge.plantuml.FileFormatOption;
import net.sourceforge.plantuml.SourceStringReader;
import net.sourceforge.plantuml.core.Diagram;
import net.sourceforge.plantuml.core.UmlSource;
import net.sourceforge.plantuml.error.PSystemError;

@RestController
@RequestMapping("/api")
@Slf4j
public class API {

    static public String getUmlSource(String source) {
        // encapsulate the UML syntax if necessary
        String uml;
        if (source.startsWith("@start")) {
            uml = source;
        } else {
            StringBuilder plantUmlSource = new StringBuilder();
            plantUmlSource.append("@startuml\n");
            plantUmlSource.append(source);
            if (source.endsWith("\n") == false) {
                plantUmlSource.append("\n");
            }
            plantUmlSource.append("@enduml");
            uml = plantUmlSource.toString();
        }
        return uml;
    }

    @PostMapping("/v1/megreflow")
    public Result megreSVG(@RequestParam("src") String src, @RequestParam("dst") String dst) throws IOException {
        String source = getUmlSource(src);
        List<BlockUml> srcblocks = new SourceStringReader(source).getBlocks();

        String destination = getUmlSource(dst);
        List<BlockUml> dstblocks = new SourceStringReader(destination).getBlocks();

        Result result = new Result();

        if (srcblocks.size() != 0 && dstblocks.size() != 0) {
            BlockUml sblock = srcblocks.get(0);
            BlockUml dblock = srcblocks.get(0);
            Diagram d1 = sblock.getDiagram();
            Diagram d2 = dblock.getDiagram();

            UmlSource s1 = d1.getSource();
            UmlSource s2 = d2.getSource();
        }
        result.setType(ResultType.ErrorMegreUML2SVG);
        return result;
    }

    @PostMapping("/v1/uml2svg")
    public Result uml2SVG(@RequestParam("data") String data) throws IOException {
        String source = getUmlSource(data);
        List<BlockUml> blocks = new SourceStringReader(source).getBlocks();
        // FileOutputStream os = new FileOutputStream(new File("./t.svg"));
        // new SourceStringReader(testUML).outputImage(os,new FileFormatOption(FileFormat.SVG));

        Result result = new Result();
        if(blocks.size() != 0) {
            for(BlockUml block: blocks) {
                Diagram diagram = block.getDiagram();
                if (diagram instanceof PSystemError) {
                    result.setType(ResultType.ErrorSource2SVG);
                    int errorLine = ((PSystemError) diagram).getLineLocation().getPosition();
                    result.setMsg("error line: " + errorLine);
                } else {
                    result.setType(ResultType.Source2SVG);
                }

                OutputStream os = new ByteArrayOutputStream();
                diagram.exportDiagram(os, 0, new FileFormatOption(FileFormat.SVG));
                result.getContent().add(os.toString());
                return result;
            }
        }

        

        result.setType(ResultType.ErrorSource2SVG);
        log.debug( ResultType.ErrorSource2SVG + data);
        return result;
    }
}